@@ -1,5 +1,5 @@
 /*
-  QtCurve (C) Craig Drummond, 2007 Craig.Drummond@lycos.co.uk
+  QtCurve (C) Craig Drummond, 2007-2008 Craig.Drummond@lycos.co.uk
 
   ----
 
@@ -30,6 +30,12 @@
 #include <QMenu>
 #include <QFileInfo>
 #include <QBoxLayout>
+#include <QGridLayout>
+#include <QTreeWidget>
+#include <QPainter>
+#include <QSettings>
+#include <KGuiItem>
+#include <KInputDialog>
 #include <klocale.h>
 #include <kcolorbutton.h>
 #include <kconfig.h>
@@ -38,6 +44,7 @@
 #include <kpushbutton.h>
 #include <kcharselect.h>
 #include <kdialog.h>
+#include <knuminput.h>
 #include <unistd.h>
 #include "config.h"
 #define CONFIG_READ
@@ -88,6 +95,110 @@ class CharSelectDialog : public KDialog
     KCharSelect *itsSelector;
 };
 
+class CGradItem : public QTreeWidgetItem
+{
+    public:
+
+    CGradItem(QTreeWidget *p, const QStringList &vals)
+        : QTreeWidgetItem(p, vals)
+    {
+        setFlags(flags()|Qt::ItemIsEditable);
+    }
+
+    virtual ~CGradItem() { }
+
+    bool operator<(const QTreeWidgetItem &i) const
+    {
+        return text(0).toDouble()<i.text(0).toDouble() ||
+               (equal(text(0).toDouble(), i.text(0).toDouble()) &&
+               text(1).toDouble()<i.text(1).toDouble());
+    }
+};
+
+static QStringList toList(const QString &str)
+{
+    QStringList lst;
+    lst.append(str);
+    return lst;
+}
+
+class CStackItem : public QTreeWidgetItem
+{
+    public:
+
+    CStackItem(QTreeWidget *p, const QString &text, int s)
+        : QTreeWidgetItem(p, toList(text)),
+          stackId(s)
+    {
+    }
+
+    bool operator<(const QTreeWidgetItem &o) const
+    {
+        return stackId<((CStackItem &)o).stackId;
+    }
+
+    int stack() { return stackId; }
+
+    private:
+
+    int stackId;
+};
+
+CGradientPreview::CGradientPreview(QWidget *p)
+                : QWidget(p)
+{
+    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
+}
+
+QSize CGradientPreview::sizeHint() const
+{
+    return QSize(64, 64);
+}
+
+QSize CGradientPreview::minimumSizeHint() const
+{
+    return sizeHint();
+}
+
+void CGradientPreview::paintEvent(QPaintEvent *)
+{
+    QRect    r(rect());
+    QPainter p(this);
+
+    if(stops.size())
+    {
+        QLinearGradient              grad(r.topLeft(), r.bottomLeft());
+        GradientCont::const_iterator it(stops.begin()),
+                                     end(stops.end());
+
+        for(; it!=end; ++it)
+        {
+            QColor col;
+            shade(color, &col, (*it).val);
+            grad.setColorAt((*it).pos, col);
+        }
+        p.fillRect(r, QBrush(grad));
+    }
+    else
+        p.fillRect(r, color);
+    p.end();
+}
+
+void CGradientPreview::setGrad(const GradientCont &s)
+{
+    stops=s;
+    repaint();
+}
+
+void CGradientPreview::setColor(const QColor &col)
+{
+    if(col!=color)
+    {
+        color=col;
+        repaint();
+    }
+}
+
 static int toInt(const QString &str)
 {
     return str.length()>1 ? str[0].unicode() : 0;
@@ -115,18 +226,22 @@ static void insertShadeEntries(QComboBox *combo, bool withDarken, bool checkRadi
     }
 }
 
-static void insertAppearanceEntries(QComboBox *combo, bool all=true)
+static void insertAppearanceEntries(QComboBox *combo, bool split=true, bool bev=true)
 {
+    for(int i=APPEARANCE_CUSTOM1; i<(APPEARANCE_CUSTOM1+QTC_NUM_CUSTOM_GRAD); ++i)
+        combo->insertItem(i, i18n("Custom gradient %1", (i-APPEARANCE_CUSTOM1)+1));
+
     combo->insertItem(APPEARANCE_FLAT, i18n("Flat"));
     combo->insertItem(APPEARANCE_RAISED, i18n("Raised"));
     combo->insertItem(APPEARANCE_DULL_GLASS, i18n("Dull glass"));
     combo->insertItem(APPEARANCE_SHINY_GLASS, i18n("Shiny glass"));
     combo->insertItem(APPEARANCE_GRADIENT, i18n("Gradient"));
     combo->insertItem(APPEARANCE_INVERTED, i18n("Inverted gradient"));
-    if(all)
+    if(split)
     {
         combo->insertItem(APPEARANCE_SPLIT_GRADIENT, i18n("Split gradient"));
-        combo->insertItem(APPEARANCE_BEVELLED, i18n("Bevelled"));
+        if(bev)
+            combo->insertItem(APPEARANCE_BEVELLED, i18n("Bevelled"));
     }
 }
 
@@ -143,7 +258,9 @@ static void insertDefBtnEntries(QComboBox *combo)
     combo->insertItem(IND_CORNER, i18n("Corner indicator"));
     combo->insertItem(IND_FONT_COLOR, i18n("Font color thin border"));
     combo->insertItem(IND_COLORED, i18n("Selected background thick border"));
-    combo->insertItem(IND_NONE, i18n("None"));
+    combo->insertItem(IND_TINT, i18n("Selected background tinting"));
+    combo->insertItem(IND_GLOW, i18n("A slight glow"));
+    combo->insertItem(IND_NONE, i18n("No indicator"));
 }
 
 static void insertScrollbarEntries(QComboBox *combo)
@@ -167,6 +284,7 @@ static void insertMouseOverEntries(QComboBox *combo)
     combo->insertItem(MO_NONE, i18n("No coloration"));
     combo->insertItem(MO_COLORED, i18n("Color border"));
     combo->insertItem(MO_PLASTIK, i18n("Plastik style"));
+    combo->insertItem(MO_GLOW, i18n("Glow"));
 }
 
 static void insertToolbarBorderEntries(QComboBox *combo)
@@ -206,12 +324,19 @@ static void insertSliderStyleEntries(QComboBox *combo)
     combo->insertItem(SLIDER_TRIANGULAR, i18n("Triangular"));
 }
 
+static void insertEColorEntries(QComboBox *combo)
+{
+    combo->insertItem(ECOLOR_BASE, i18n("Base color"));
+    combo->insertItem(ECOLOR_BACKGROUND, i18n("Background color"));
+    combo->insertItem(ECOLOR_DARK, i18n("Darkened background color"));
+}
+
 QtCurveConfig::QtCurveConfig(QWidget *parent)
              : QWidget(parent),
                exportDialog(NULL)
 {
     setupUi(this);
-    titleLabel->setText("QtCurve " VERSION " - (C) Craig Drummond, 2003-2007");
+    titleLabel->setText("QtCurve " VERSION " - (C) Craig Drummond, 2003-2008");
     insertShadeEntries(shadeSliders, false);
     insertShadeEntries(shadeMenubars, true);
     insertShadeEntries(shadeCheckRadio, false, true);
@@ -221,8 +346,13 @@ QtCurveConfig::QtCurveConfig(QWidget *parent)
     insertAppearanceEntries(lvAppearance);
     insertAppearanceEntries(sliderAppearance);
     insertAppearanceEntries(tabAppearance, false);
+    insertAppearanceEntries(activeTabAppearance, false);
     insertAppearanceEntries(progressAppearance);
+    insertAppearanceEntries(progressGrooveAppearance);
     insertAppearanceEntries(menuitemAppearance);
+    insertAppearanceEntries(titlebarAppearance, true, false);
+    insertAppearanceEntries(selectionAppearance, true, false);
+    insertAppearanceEntries(menuStripeAppearance, true, false);
     insertLineEntries(handles, false);
     insertLineEntries(sliderThumbs, true);
     insertLineEntries(toolbarSeparators, true);
@@ -236,11 +366,14 @@ QtCurveConfig::QtCurveConfig(QWidget *parent)
     insertShadingEntries(shading);
     insertStripeEntries(stripedProgress);
     insertSliderStyleEntries(sliderStyle);
+    insertEColorEntries(progressGrooveColor);
 
     highlightFactor->setRange(MIN_HIGHLIGHT_FACTOR, MAX_HIGHLIGHT_FACTOR);
     highlightFactor->setValue(((int)(DEFAULT_HIGHLIGHT_FACTOR*100))-100);
 
     connect(lighterPopupMenuBgnd, SIGNAL(toggled(bool)), SLOT(updateChanged()));
+    connect(menuStripe, SIGNAL(toggled(bool)), SLOT(updateChanged()));
+    connect(menuStripeAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(round, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(toolbarBorders, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(sliderThumbs, SIGNAL(activated(int)), SLOT(updateChanged()));
@@ -250,28 +383,35 @@ QtCurveConfig::QtCurveConfig(QWidget *parent)
     connect(stripedProgress, SIGNAL(activated(int)), SLOT(stripedProgressChanged()));
     connect(animatedProgress, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(embolden, SIGNAL(toggled(bool)), SLOT(emboldenToggled()));
-    connect(defBtnIndicator, SIGNAL(activated(int)), SLOT(dbiChanged()));
+    connect(defBtnIndicator, SIGNAL(activated(int)), SLOT(defBtnIndicatorChanged()));
     connect(highlightTab, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(menubarAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(toolbarAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(lvAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(sliderAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
-    connect(tabAppearance, SIGNAL(activated(int)), SLOT(tabAppearanceChanged()));
+    connect(tabAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
+    connect(activeTabAppearance, SIGNAL(activated(int)), SLOT(activeTabAppearanceChanged()));
     connect(toolbarSeparators, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(splitters, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(fixParentlessDialogs, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(fillSlider, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(sliderStyle, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(roundMbTopOnly, SIGNAL(toggled(bool)), SLOT(updateChanged()));
-    connect(gradientPbGroove, SIGNAL(toggled(bool)), SLOT(updateChanged()));
+    connect(fillProgress, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(darkerBorders, SIGNAL(toggled(bool)), SLOT(updateChanged()));
+    connect(comboSplitter, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(vArrows, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(xCheck, SIGNAL(toggled(bool)), SLOT(updateChanged()));
+    connect(crHighlight, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(colorSelTab, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(stdSidebarButtons, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(borderMenuitems, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(progressAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
+    connect(progressGrooveAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
+    connect(progressGrooveColor, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(menuitemAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
+    connect(titlebarAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
+    connect(selectionAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
     connect(shadeCheckRadio, SIGNAL(activated(int)), SLOT(shadeCheckRadioChanged()));
     connect(customCheckRadioColor, SIGNAL(changed(const QColor &)), SLOT(updateChanged()));
 
@@ -282,8 +422,8 @@ QtCurveConfig::QtCurveConfig(QWidget *parent)
 #endif
     connect(lvLines, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(drawStatusBarFrames, SIGNAL(toggled(bool)), SLOT(updateChanged()));
-    connect(buttonEffect, SIGNAL(activated(int)), SLOT(updateChanged()));
-    connect(coloredMouseOver, SIGNAL(activated(int)), SLOT(updateChanged()));
+    connect(buttonEffect, SIGNAL(activated(int)), SLOT(buttonEffectChanged()));
+    connect(coloredMouseOver, SIGNAL(activated(int)), SLOT(coloredMouseOverChanged()));
     connect(menubarMouseOver, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(shadeMenubarOnlyWhenActive, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(thinnerMenuItems, SIGNAL(toggled(bool)), SLOT(updateChanged()));
@@ -295,8 +435,11 @@ QtCurveConfig::QtCurveConfig(QWidget *parent)
     connect(shadeMenubars, SIGNAL(activated(int)), SLOT(shadeMenubarsChanged()));
     connect(highlightFactor, SIGNAL(valueChanged(int)), SLOT(updateChanged()));
     connect(scrollbarType, SIGNAL(activated(int)), SLOT(updateChanged()));
-    connect(shading, SIGNAL(activated(int)), SLOT(updateChanged()));
+    connect(shading, SIGNAL(activated(int)), SLOT(shadingChanged()));
     connect(gtkScrollViews, SIGNAL(toggled(bool)), SLOT(updateChanged()));
+    connect(squareScrollViews, SIGNAL(toggled(bool)), SLOT(updateChanged()));
+    connect(highlightScrollViews, SIGNAL(toggled(bool)), SLOT(updateChanged()));
+    connect(sunkenScrollViews, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(gtkComboMenus, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(gtkButtonOrder, SIGNAL(toggled(bool)), SLOT(updateChanged()));
     connect(mapKdeIcons, SIGNAL(toggled(bool)), SLOT(updateChanged()));
@@ -308,6 +451,7 @@ QtCurveConfig::QtCurveConfig(QWidget *parent)
     if(!readConfig(NULL, &currentStyle, &defaultStyle))
         currentStyle=defaultStyle;
 
+    setupShadesTab();
     setWidgetOptions(currentStyle);
 
     QMenu *menu=new QMenu(this),
@@ -322,6 +466,8 @@ QtCurveConfig::QtCurveConfig(QWidget *parent)
     menu->addSeparator();
     menu->addAction(i18n("Export Theme..."), this, SLOT(exportTheme()));
     loadStyles(subMenu);
+    setupGradientsTab();
+    setupStack();
 }
 
 QtCurveConfig::~QtCurveConfig()
@@ -377,14 +523,39 @@ void QtCurveConfig::setStyle(QAction *s)
 void QtCurveConfig::emboldenToggled()
 {
     if(!embolden->isChecked() && IND_NONE==defBtnIndicator->currentIndex())
-        defBtnIndicator->setCurrentIndex(IND_COLORED);
+        defBtnIndicator->setCurrentIndex(IND_TINT);
     updateChanged();
 }
 
-void QtCurveConfig::dbiChanged()
+void QtCurveConfig::defBtnIndicatorChanged()
 {
     if(IND_NONE==defBtnIndicator->currentIndex() && !embolden->isChecked())
         embolden->setChecked(true);
+    else if(IND_GLOW==defBtnIndicator->currentIndex() && EFFECT_NONE==buttonEffect->currentIndex())
+        buttonEffect->setCurrentIndex(EFFECT_SHADOW);
+
+    updateChanged();
+}
+
+void QtCurveConfig::buttonEffectChanged()
+{
+    if(EFFECT_NONE==buttonEffect->currentIndex())
+    {
+        if(IND_GLOW==defBtnIndicator->currentIndex())
+            defBtnIndicator->setCurrentIndex(IND_TINT);
+        if(MO_GLOW==coloredMouseOver->currentIndex())
+            coloredMouseOver->setCurrentIndex(MO_PLASTIK);
+    }
+
+    updateChanged();
+}
+
+void QtCurveConfig::coloredMouseOverChanged()
+{
+    if(MO_GLOW==coloredMouseOver->currentIndex() &&
+       EFFECT_NONE==buttonEffect->currentIndex())
+        buttonEffect->setCurrentIndex(EFFECT_SHADOW);
+
     updateChanged();
 }
 
@@ -421,11 +592,19 @@ void QtCurveConfig::stripedProgressChanged()
     updateChanged();
 }
 
-void QtCurveConfig::tabAppearanceChanged()
+void QtCurveConfig::activeTabAppearanceChanged()
 {
-    if(colorSelTab->isChecked() && APPEARANCE_GRADIENT!=tabAppearance->currentIndex())
+    int current(activeTabAppearance->currentIndex());
+
+    if(colorSelTab->isChecked() && APPEARANCE_GRADIENT!=current && APPEARANCE_INVERTED!=current)
         colorSelTab->setChecked(false);
-    colorSelTab->setEnabled(APPEARANCE_GRADIENT==tabAppearance->currentIndex());
+    colorSelTab->setEnabled(APPEARANCE_GRADIENT==current || APPEARANCE_INVERTED==current);
+    updateChanged();
+}
+
+void QtCurveConfig::shadingChanged()
+{
+    ::shading=(EShading)shading->currentIndex();
     updateChanged();
 }
 
@@ -438,6 +617,335 @@ void QtCurveConfig::passwordCharClicked()
         setPasswordChar(dlg.currentChar());
 }
 
+void QtCurveConfig::setupStack()
+{
+    int i=0;
+    CStackItem *first=new CStackItem(stackList, i18n("General"), i++);
+    new CStackItem(stackList, i18n("Splitters"), i++);
+    new CStackItem(stackList, i18n("Sliders and Scrollbars"), i++);
+    new CStackItem(stackList, i18n("Progressbars"), i++);
+    new CStackItem(stackList, i18n("Default Button"),i++);
+    new CStackItem(stackList, i18n("Mouse-over"), i++);
+    new CStackItem(stackList, i18n("Listviews"), i++);
+    new CStackItem(stackList, i18n("Scrollviews"), i++);
+    new CStackItem(stackList, i18n("Tabs"), i++);
+    new CStackItem(stackList, i18n("Checks and Radios"), i++);
+    new CStackItem(stackList, i18n("Titlebars"), i++);
+    new CStackItem(stackList, i18n("Menus and Toolbars"), i++);
+    new CStackItem(stackList, i18n("Advanced Settings"), i++);
+    new CStackItem(stackList, i18n("Custom Gradients"), i++);
+    new CStackItem(stackList, i18n("Custom Shades"), i++);
+
+    stackList->setSelectionMode(QAbstractItemView::SingleSelection);
+    first->setSelected(true);
+    connect(stackList, SIGNAL(itemSelectionChanged()), SLOT(changeStack()));
+}
+
+void QtCurveConfig::changeStack()
+{
+    CStackItem *item=(CStackItem *)(stackList->currentItem());
+
+    if(item && !item->isSelected())
+        item->setSelected(true);
+
+    if(item)
+        stack->setCurrentIndex(item->stack());
+}
+
+void QtCurveConfig::gradChanged(int i)
+{
+    CustomGradientCont::const_iterator it(customGradient.find((EAppearance)i));
+
+    gradStops->clear();
+
+    if(it!=customGradient.end())
+    {
+        gradPreview->setGrad((*it).second.grad);
+        gradLightBorder->setChecked((*it).second.lightBorder);
+
+        GradientCont::const_iterator git((*it).second.grad.begin()),
+                                     gend((*it).second.grad.end());
+
+        for(; git!=gend; ++git)
+        {
+            QStringList details;
+
+            details << QString().setNum((*git).pos*100.0)
+                    << QString().setNum((*git).val*100.0);
+
+            new CGradItem(gradStops, details);
+        }
+
+        gradStops->sortItems(0, Qt::AscendingOrder);
+    }
+    else
+    {
+        gradPreview->setGrad(GradientCont());
+        gradLightBorder->setChecked(false);
+    }
+
+    gradLightBorder->setEnabled(APPEARANCE_SUNKEN!=i);
+}
+
+static double prev=0.0;
+
+void QtCurveConfig::editItem(QTreeWidgetItem *i, int col)
+{
+    bool   ok;
+    prev=i->text(col).toDouble(&ok);
+    if(!ok)
+        prev=0.0;
+
+    gradStops->editItem(i, col);
+}
+
+void QtCurveConfig::itemChanged(QTreeWidgetItem *i, int col)
+{
+    bool   ok;
+    double val=i->text(col).toDouble(&ok)/100.0;
+
+    if(ok && equal(val, prev))
+        return;
+
+    if(!ok || (0==col && (val<0.0 || val>1.0)) || (1==col && (val<0.0 || val>2.0)))
+        i->setText(col, QString().setNum(prev));
+    else
+    {
+        double other=i->text(col ? 0 : 1).toDouble(&ok)/100.0;
+
+        CustomGradientCont::iterator it=customGradient.find((EAppearance)gradCombo->currentIndex());
+
+        if(it!=customGradient.end())
+        {
+            (*it).second.grad.erase(Gradient(col ? other : prev, col ? prev : other));
+            (*it).second.grad.insert(Gradient(col ? other : val, col ? val : other));
+            gradPreview->setGrad((*it).second.grad);
+            i->setText(col, QString().setNum(val*100.0));
+            emit changed(true);
+        }
+    }
+}
+
+void QtCurveConfig::addGradStop()
+{
+    bool added(false);
+
+    CustomGradientCont::iterator cg=customGradient.find((EAppearance)gradCombo->currentIndex());
+
+    if(cg==customGradient.end())
+    {
+        CustomGradient cust;
+
+        cust.lightBorder=gradLightBorder->isChecked();
+        cust.grad.insert(Gradient(stopPosition->value()/100.0, stopValue->value()/100.0));
+        customGradient[(EAppearance)gradCombo->currentIndex()]=cust;
+        added=true;
+        gradChanged(gradCombo->currentIndex());
+        emit changed(true);
+    }
+    else
+    {
+        GradientCont::const_iterator it((*cg).second.grad.begin()),
+                                     end((*cg).second.grad.end());
+        double                       pos(stopPosition->value()/100.0),
+                                     val(stopValue->value()/100.0);
+
+        for(; it!=end; ++it)
+            if(equal(pos, (*it).pos))
+                if(equal(val, (*it).val))
+                    return;
+                else
+                {
+                    (*cg).second.grad.erase(it);
+                    break;
+                }
+
+        unsigned int b4=(*cg).second.grad.size();
+        (*cg).second.grad.insert(Gradient(pos, val));
+        if((*cg).second.grad.size()!=b4)
+        {
+            gradPreview->setGrad((*cg).second.grad);
+
+            QStringList details;
+
+            details << QString().setNum(pos*100.0)
+                    << QString().setNum(val*100.0);
+
+            QTreeWidgetItem *i=new CGradItem(gradStops, details);
+
+            gradStops->setCurrentItem(i);
+            gradStops->sortItems(0, Qt::AscendingOrder);
+        }
+    }
+}
+
+void QtCurveConfig::removeGradStop()
+{
+    QTreeWidgetItem *cur=gradStops->currentItem();
+
+    if(cur)
+    {
+        QTreeWidgetItem *next=gradStops->itemBelow(cur);
+
+        if(!next)
+            next=gradStops->itemAbove(cur);
+
+        CustomGradientCont::iterator it=customGradient.find((EAppearance)gradCombo->currentIndex());
+
+        if(it!=customGradient.end())
+        {
+            bool   ok;
+            double pos=cur->text(0).toDouble(&ok)/100.0,
+                   val=cur->text(1).toDouble(&ok)/100.0;
+
+            (*it).second.grad.erase(Gradient(pos, val));
+            gradPreview->setGrad((*it).second.grad);
+            emit changed(true);
+
+            delete cur;
+            if(next)
+                gradStops->setCurrentItem(next);
+        }
+    }
+}
+
+void QtCurveConfig::updateGradStop()
+{
+    QTreeWidgetItem *i=gradStops->selectedItems().size() ? *(gradStops->selectedItems().begin()) : 0L;
+
+    CustomGradientCont::iterator cg=customGradient.find((EAppearance)gradCombo->currentIndex());
+
+    if(i)
+    {
+        double curPos=i->text(0).toDouble()/100.0,
+               curVal=i->text(1).toDouble()/100.0,
+               newPos(stopPosition->value()/100.0),
+               newVal(stopValue->value()/100.0);
+
+        if(!equal(newPos, curPos) || !equal(newVal, curVal))
+        {
+            (*cg).second.grad.erase(Gradient(curPos, curVal));
+            (*cg).second.grad.insert(Gradient(newPos, newVal));
+
+            i->setText(0, QString().setNum(stopPosition->value()));
+            i->setText(1, QString().setNum(stopValue->value()));
+            gradPreview->setGrad((*cg).second.grad);
+            emit changed(true);
+        }
+    }
+    else
+        addGradStop();
+}
+
+void QtCurveConfig::stopSelected()
+{
+    QTreeWidgetItem *i=gradStops->selectedItems().size() ? *(gradStops->selectedItems().begin()) : 0L;
+
+    removeButton->setEnabled(i);
+    updateButton->setEnabled(i);
+
+    if(i)
+    {
+        stopPosition->setValue(i->text(0).toInt());
+        stopValue->setValue(i->text(1).toInt());
+    }
+    else
+    {
+        stopPosition->setValue(0);
+        stopValue->setValue(0);
+    }
+}
+
+void QtCurveConfig::setupGradientsTab()
+{
+    for(int i=APPEARANCE_CUSTOM1; i<(APPEARANCE_CUSTOM1+QTC_NUM_CUSTOM_GRAD); ++i)
+        gradCombo->insertItem(i-APPEARANCE_CUSTOM1, i18n("Custom gradient %1", (i-APPEARANCE_CUSTOM1)+1));
+    gradCombo->insertItem(APPEARANCE_CUSTOM1+QTC_NUM_CUSTOM_GRAD, i18n("Custom sunken gradient"));
+
+    gradCombo->setCurrentIndex(APPEARANCE_CUSTOM1);
+
+    gradPreview=new CGradientPreview(previewWidgetContainer);
+    QBoxLayout *layout=new QBoxLayout(QBoxLayout::TopToBottom, previewWidgetContainer);
+    layout->addWidget(gradPreview);
+    layout->setMargin(0);
+    layout->setSpacing(0);
+    QColor col(palette().color(QPalette::Active, QPalette::Button));
+    previewColor->setColor(col);
+    gradPreview->setColor(col);
+    gradChanged(APPEARANCE_CUSTOM1);
+    addButton->setGuiItem(KGuiItem(i18n("Add"), "list-add"));
+    removeButton->setGuiItem(KGuiItem(i18n("Remove"), "list-remove"));
+    updateButton->setGuiItem(KGuiItem(i18n("Update"), "dialog-ok"));
+
+    stopPosition->setRange(0, 100, 5);
+    stopValue->setRange(0, 200, 5);
+    removeButton->setEnabled(false);
+    updateButton->setEnabled(false);
+    connect(gradCombo, SIGNAL(currentIndexChanged(int)), SLOT(gradChanged(int)));
+    connect(previewColor, SIGNAL(changed(const QColor &)), gradPreview, SLOT(setColor(const QColor &)));
+    connect(gradStops, SIGNAL(itemDoubleClicked(QTreeWidgetItem *, int)), SLOT(editItem(QTreeWidgetItem *, int)));
+    connect(gradStops, SIGNAL(itemChanged(QTreeWidgetItem *, int)), SLOT(itemChanged(QTreeWidgetItem *, int)));
+    connect(addButton, SIGNAL(clicked(bool)), SLOT(addGradStop()));
+    connect(removeButton, SIGNAL(clicked(bool)), SLOT(removeGradStop()));
+    connect(updateButton, SIGNAL(clicked(bool)), SLOT(updateGradStop()));
+    connect(gradStops, SIGNAL(itemSelectionChanged()), SLOT(stopSelected()));
+}
+
+void QtCurveConfig::setupShadesTab()
+{
+    int shade(0);
+
+    setupShade(shade0, shade++);
+    setupShade(shade1, shade++);
+    setupShade(shade2, shade++);
+    setupShade(shade3, shade++);
+    setupShade(shade4, shade++);
+    setupShade(shade5, shade++);
+    connect(customShading, SIGNAL(toggled(bool)), SLOT(updateChanged()));
+}
+
+void QtCurveConfig::setupShade(KDoubleNumInput *w, int shade)
+{
+    w->setRange(0.0, 2.0, 0.05, false);
+    connect(w, SIGNAL(valueChanged(double)), SLOT(updateChanged()));
+    shadeVals[shade]=w;
+}
+
+void QtCurveConfig::populateShades(const Options &opts)
+{
+    QTC_SHADES
+    int contrast=QSettings(QLatin1String("Trolltech")).value("/Qt/KDE/contrast", 7).toInt();
+
+    if(contrast<0 || contrast>10)
+        contrast=7;
+
+    customShading->setChecked(opts.customShades.size());
+
+    for(int i=0; i<NUM_STD_SHADES; ++i)
+        shadeVals[i]->setValue(opts.customShades.size()
+                                  ? opts.customShades[i]
+                                  : shades[SHADING_SIMPLE==shading->currentIndex()
+                                            ? 1 : 0]
+                                          [contrast]
+                                          [i]);
+}
+
+bool QtCurveConfig::diffShades(const Options &opts)
+{
+    if( (0==opts.customShades.size() && customShading->isChecked()) ||
+        (opts.customShades.size() && !customShading->isChecked()) )
+        return true;
+
+    if(customShading->isChecked())
+    {
+        for(int i=0; i<NUM_STD_SHADES; ++i)
+            if(!equal(shadeVals[i]->value(), opts.customShades[i]))
+                return true;
+    }
+
+    return false;
+}
+
 void QtCurveConfig::setPasswordChar(int ch)
 {
     QString     str;
@@ -528,6 +1036,8 @@ void QtCurveConfig::setOptions(Options &opts)
     opts.animatedProgress=animatedProgress->isChecked();
     opts.stripedProgress=(EStripe)stripedProgress->currentIndex();
     opts.lighterPopupMenuBgnd=lighterPopupMenuBgnd->isChecked();
+    opts.menuStripe=menuStripe->isChecked();
+    opts.menuStripeAppearance=(EAppearance)menuStripeAppearance->currentIndex();
     opts.embolden=embolden->isChecked();
     opts.scrollbarType=(EScrollbar)scrollbarType->currentIndex();
     opts.defBtnIndicator=(EDefBtnIndicator)defBtnIndicator->currentIndex();
@@ -541,6 +1051,7 @@ void QtCurveConfig::setOptions(Options &opts)
     opts.lvAppearance=(EAppearance)lvAppearance->currentIndex();
     opts.sliderAppearance=(EAppearance)sliderAppearance->currentIndex();
     opts.tabAppearance=(EAppearance)tabAppearance->currentIndex();
+    opts.activeTabAppearance=(EAppearance)activeTabAppearance->currentIndex();
     opts.toolbarSeparators=(ELine)toolbarSeparators->currentIndex();
     opts.splitters=(ELine)splitters->currentIndex();
     opts.customSlidersColor=customSlidersColor->color();
@@ -552,25 +1063,44 @@ void QtCurveConfig::setOptions(Options &opts)
     opts.fillSlider=fillSlider->isChecked();
     opts.sliderStyle=(ESliderStyle)sliderStyle->currentIndex();
     opts.roundMbTopOnly=roundMbTopOnly->isChecked();
-    opts.gradientPbGroove=gradientPbGroove->isChecked();
+    opts.fillProgress=fillProgress->isChecked();
     opts.darkerBorders=darkerBorders->isChecked();
+    opts.comboSplitter=comboSplitter->isChecked();
     opts.vArrows=vArrows->isChecked();
     opts.xCheck=xCheck->isChecked();
+    opts.crHighlight=crHighlight->isChecked();
     opts.colorSelTab=colorSelTab->isChecked();
     opts.stdSidebarButtons=stdSidebarButtons->isChecked();
     opts.borderMenuitems=borderMenuitems->isChecked();
     opts.progressAppearance=(EAppearance)progressAppearance->currentIndex();
+    opts.progressGrooveAppearance=(EAppearance)progressGrooveAppearance->currentIndex();
+    opts.progressGrooveColor=(EColor)progressGrooveColor->currentIndex();
     opts.menuitemAppearance=(EAppearance)menuitemAppearance->currentIndex();
+    opts.titlebarAppearance=(EAppearance)titlebarAppearance->currentIndex();
+    opts.selectionAppearance=(EAppearance)selectionAppearance->currentIndex();
     opts.shadeCheckRadio=(EShade)shadeCheckRadio->currentIndex();
     opts.customCheckRadioColor=customCheckRadioColor->color();
     opts.shading=(EShading)shading->currentIndex();
     opts.gtkScrollViews=gtkScrollViews->isChecked();
+    opts.highlightScrollViews=highlightScrollViews->isChecked();
+    opts.squareScrollViews=squareScrollViews->isChecked();
+    opts.sunkenScrollViews=sunkenScrollViews->isChecked();
     opts.gtkComboMenus=gtkComboMenus->isChecked();
     opts.gtkButtonOrder=gtkButtonOrder->isChecked();
     opts.mapKdeIcons=mapKdeIcons->isChecked();
     opts.passwordChar=toInt(passwordChar->text());
     opts.framelessGroupBoxes=framelessGroupBoxes->isChecked();
     opts.inactiveHighlight=inactiveHighlight->isChecked();
+    opts.customGradient=customGradient;
+
+    if(customShading->isChecked())
+    {
+        opts.customShades.resize(NUM_STD_SHADES);
+        for(int i=0; i<NUM_STD_SHADES; ++i)
+            opts.customShades[i]=shadeVals[i]->value();
+    }
+    else
+        opts.customShades.clear();
 }
 
 void QtCurveConfig::setWidgetOptions(const Options &opts)
@@ -578,6 +1108,8 @@ void QtCurveConfig::setWidgetOptions(const Options &opts)
     round->setCurrentIndex(opts.round);
     scrollbarType->setCurrentIndex(opts.scrollbarType);
     lighterPopupMenuBgnd->setChecked(opts.lighterPopupMenuBgnd);
+    menuStripe->setChecked(opts.menuStripe);
+    menuStripeAppearance->setCurrentIndex(opts.menuStripeAppearance);
     toolbarBorders->setCurrentIndex(opts.toolbarBorders);
     sliderThumbs->setCurrentIndex(opts.sliderThumbs);
     handles->setCurrentIndex(opts.handles);
@@ -603,6 +1135,7 @@ void QtCurveConfig::setWidgetOptions(const Options &opts)
     lvAppearance->setCurrentIndex(opts.lvAppearance);
     sliderAppearance->setCurrentIndex(opts.sliderAppearance);
     tabAppearance->setCurrentIndex(opts.tabAppearance);
+    activeTabAppearance->setCurrentIndex(opts.activeTabAppearance);
     toolbarSeparators->setCurrentIndex(opts.toolbarSeparators);
     splitters->setCurrentIndex(opts.splitters);
     shadeSliders->setCurrentIndex(opts.shadeSliders);
@@ -625,26 +1158,39 @@ void QtCurveConfig::setWidgetOptions(const Options &opts)
     fillSlider->setChecked(opts.fillSlider);
     sliderStyle->setCurrentIndex(opts.sliderStyle);
     roundMbTopOnly->setChecked(opts.roundMbTopOnly);
-    gradientPbGroove->setChecked(opts.gradientPbGroove);
+    fillProgress->setChecked(opts.fillProgress);
     darkerBorders->setChecked(opts.darkerBorders);
+    comboSplitter->setChecked(opts.comboSplitter);
     vArrows->setChecked(opts.vArrows);
     xCheck->setChecked(opts.xCheck);
+    crHighlight->setChecked(opts.crHighlight);
     colorSelTab->setChecked(opts.colorSelTab);
     stdSidebarButtons->setChecked(opts.stdSidebarButtons);
     borderMenuitems->setChecked(opts.borderMenuitems);
     progressAppearance->setCurrentIndex(opts.progressAppearance);
+    progressGrooveAppearance->setCurrentIndex(opts.progressGrooveAppearance);
+    progressGrooveColor->setCurrentIndex(opts.progressGrooveColor);
     menuitemAppearance->setCurrentIndex(opts.menuitemAppearance);
+    titlebarAppearance->setCurrentIndex(opts.titlebarAppearance);
+    selectionAppearance->setCurrentIndex(opts.selectionAppearance);
     shadeCheckRadio->setCurrentIndex(opts.shadeCheckRadio);
     customCheckRadioColor->setColor(opts.customCheckRadioColor);
 
     shading->setCurrentIndex(opts.shading);
     gtkScrollViews->setChecked(opts.gtkScrollViews);
+    highlightScrollViews->setChecked(opts.highlightScrollViews);
+    squareScrollViews->setChecked(opts.squareScrollViews);
+    sunkenScrollViews->setChecked(opts.sunkenScrollViews);
     gtkComboMenus->setChecked(opts.gtkComboMenus);
     gtkButtonOrder->setChecked(opts.gtkButtonOrder);
     mapKdeIcons->setChecked(opts.mapKdeIcons);
     setPasswordChar(opts.passwordChar);
     framelessGroupBoxes->setChecked(opts.framelessGroupBoxes);
     inactiveHighlight->setChecked(opts.inactiveHighlight);
+    customGradient=opts.customGradient;
+    gradCombo->setCurrentIndex(APPEARANCE_CUSTOM1);
+
+    populateShades(opts);
 }
 
 bool QtCurveConfig::settingsChanged()
@@ -666,14 +1212,18 @@ bool QtCurveConfig::settingsChanged()
          animatedProgress->isChecked()!=currentStyle.animatedProgress ||
          stripedProgress->currentIndex()!=currentStyle.stripedProgress ||
          lighterPopupMenuBgnd->isChecked()!=currentStyle.lighterPopupMenuBgnd ||
+         menuStripe->isChecked()!=currentStyle.menuStripe ||
+         menuStripeAppearance->currentIndex()!=currentStyle.menuStripeAppearance ||
          embolden->isChecked()!=currentStyle.embolden ||
          fillSlider->isChecked()!=currentStyle.fillSlider ||
          sliderStyle->currentIndex()!=currentStyle.sliderStyle ||
          roundMbTopOnly->isChecked()!=currentStyle.roundMbTopOnly ||
-         gradientPbGroove->isChecked()!=currentStyle.gradientPbGroove ||
+         fillProgress->isChecked()!=currentStyle.fillProgress ||
          darkerBorders->isChecked()!=currentStyle.darkerBorders ||
+         comboSplitter->isChecked()!=currentStyle.comboSplitter ||
          vArrows->isChecked()!=currentStyle.vArrows ||
          xCheck->isChecked()!=currentStyle.xCheck ||
+         crHighlight->isChecked()!=currentStyle.crHighlight ||
          colorSelTab->isChecked()!=currentStyle.colorSelTab ||
          stdSidebarButtons->isChecked()!=currentStyle.stdSidebarButtons ||
          borderMenuitems->isChecked()!=currentStyle.borderMenuitems ||
@@ -690,13 +1240,21 @@ bool QtCurveConfig::settingsChanged()
          lvAppearance->currentIndex()!=currentStyle.lvAppearance ||
          sliderAppearance->currentIndex()!=currentStyle.sliderAppearance ||
          tabAppearance->currentIndex()!=currentStyle.tabAppearance ||
+         activeTabAppearance->currentIndex()!=currentStyle.activeTabAppearance ||
          progressAppearance->currentIndex()!=currentStyle.progressAppearance ||
+         progressGrooveAppearance->currentIndex()!=currentStyle.progressGrooveAppearance ||
+         progressGrooveColor->currentIndex()!=currentStyle.progressGrooveColor ||
          menuitemAppearance->currentIndex()!=currentStyle.menuitemAppearance ||
+         titlebarAppearance->currentIndex()!=currentStyle.titlebarAppearance ||
+         selectionAppearance->currentIndex()!=currentStyle.selectionAppearance ||
          toolbarSeparators->currentIndex()!=currentStyle.toolbarSeparators ||
          splitters->currentIndex()!=currentStyle.splitters ||
 
          shading->currentIndex()!=(int)currentStyle.shading ||
          gtkScrollViews->isChecked()!=currentStyle.gtkScrollViews ||
+         highlightScrollViews->isChecked()!=currentStyle.highlightScrollViews ||
+         squareScrollViews->isChecked()!=currentStyle.squareScrollViews ||
+         sunkenScrollViews->isChecked()!=currentStyle.sunkenScrollViews ||
          gtkComboMenus->isChecked()!=currentStyle.gtkComboMenus ||
          gtkButtonOrder->isChecked()!=currentStyle.gtkButtonOrder ||
          mapKdeIcons->isChecked()!=currentStyle.mapKdeIcons ||
@@ -716,7 +1274,11 @@ bool QtCurveConfig::settingsChanged()
          (customMenuTextColor->isChecked() &&
                customMenuNormTextColor->color()!=currentStyle.customMenuNormTextColor) ||
          (customMenuTextColor->isChecked() &&
-               customMenuSelTextColor->color()!=currentStyle.customMenuSelTextColor);
+               customMenuSelTextColor->color()!=currentStyle.customMenuSelTextColor) ||
+
+         customGradient!=currentStyle.customGradient ||
+
+         diffShades(currentStyle);
 }
 
 #include "qtcurveconfig.moc"
